'''
Created on 11 Oct 2016

@author: jdrumgoole
'''

__version__       = "1.0.4alpha"
__programName__   = "MUGAlyser"
#
# Schema version 1.1 : Date mapped the new Group fields so they appear as date time.
#
__schemaVersion__ = "1.1"
